module Husky

  class Entity

    class << self

      def wrap(items)
        items.map { |item| build(item) }
      end

      def build(data = nil)
        instance = allocate
        instance.construct(data)
        instance
      end

      def attr_required(*args)
        args.each do |arg|
          self.class_eval("def #{arg};@#{arg};end") unless respond_to? arg
          required << arg
        end
      end

      attr_writer :required

      def required
        @required ||= []
      end

    end

    attr_reader :data

    def construct(data)
      if data.nil?
        # do nothing
      elsif data.is_a? Hash
        data.each_pair do |key, value|
          instance_variable_set("@#{key}", value)

          (class << self; self; end).class_eval do
            attr_reader key.to_sym
          end
        end
      else
        @data = data
      end
    end

    def set_basic_attributes_from_hash(hash)
      hash.each_pair do |key, value|
        unless value.respond_to? :each
          instance_variable_set("@#{key}", value)
        end
      end

      hash.each_pair do |key, value|
        unless value.respond_to? :each
          self.class.send(:define_method, key) do
            instance_variable_get("@#{key}")
          end
        end
      end
    end

    def valid?
      self.class.required.all? { |attr| !self.send(attr).nil? }
    end

    def method_missing(method, *args, &block)
      raise "#{method} called on @data, but @data is nil." if data.nil?
      data.send(method, *args, &block)
    end

  end

end